#pragma once
#include <vector>
#include "data_types.h"
#include "imgui.h"
#include "request_task.h"

inline std::vector<std::function<void()>> g_tasks;

#define FUTURE_PARAM(task_type, name) \
    inline std::future<task_type::result_type> name##_future; \
    inline task_type::result_type name##_; \
    inline void update_##name() { \
        if (name##_future.valid()) { \
            if (name##_future.wait_for(std::chrono::seconds(0)) == std::future_status::ready) { \
                name##_ = name##_future.get(); \
            } \
        } else { \
            name##_future = task_executor<task_type>::run_task(); \
        } \
    } \
    struct name##_task_register { \
        name##_task_register() { \
            g_tasks.emplace_back(update_##name); \
        } \
    } name##_task_register_instance;

FUTURE_PARAM(get_fan_info_task, fan_infos)
FUTURE_PARAM(get_fan_mode, fan_mode)
FUTURE_PARAM(get_running_time_task, running_time)
FUTURE_PARAM(get_psu_info_task, psu_status)

inline void update_infos() {
    for (const auto& task : g_tasks) {
        task();
    }
}

inline void draw_running_time() {
    ImGui::Text("Server running time: %d days %d hours", running_time_.days, running_time_.hours);
}

inline ImU32 get_fan_status_color(fan_status status) {
    switch (status) {
        case fan_status::absent:
            return IM_COL32(255, 0, 0, 255);
        case fan_status::critical:
            return IM_COL32(255, 255, 0, 255);
        case fan_status::non_critical:
            return IM_COL32(0, 255, 0, 255);
        default:
            return IM_COL32(255, 255, 255, 255);
    }
}

inline void draw_fan_infos() {
    if (ImGui::Checkbox("Manual Mode", &fan_mode_.manual_mode))
        task_executor<set_fan_mode_task>::run_task(fan_mode_.manual_mode);
    for (auto& fan_info : fan_infos_) {
        // 根据风扇状态绘制不同的颜色 和 启用/禁用输入框
        ImGui::PushStyleColor(ImGuiCol_Text, get_fan_status_color(fan_info.status));
        {
            ImGui::BeginDisabled(fan_info.status != fan_status::non_critical);
            {
                ImGui::Text("Fan %d: %d RPM", fan_info.id, fan_info.speed_rpm);
                std::string label = "Speed##" + std::to_string(fan_info.id);
                ImGui::SliderInt(label.c_str(), &fan_info.speed_percent, 1, 100);
                if (ImGui::IsItemDeactivatedAfterEdit()) {
                    // 更新风扇转速
                    fan_info.speed_percent = std::clamp(fan_info.speed_percent, 1, 100);
                    task_executor<set_fan_speed_task>::run_task(fan_info.id, fan_info.speed_percent);
                }
            }
            ImGui::EndDisabled();
        }
        ImGui::PopStyleColor();
    }
}

inline void draw_psu_infos() {
    // 将psu信息绘制成表格
    ImGui::Columns(9, "psu_status");
    ImGui::Separator();
    ImGui::Text("ID");
    ImGui::NextColumn();
    ImGui::Text("Present");
    ImGui::NextColumn();
    ImGui::Text("Power Status");
    ImGui::NextColumn();
    ImGui::Text("Temperature");
    ImGui::NextColumn();
    ImGui::Text("Power In Watts");
    ImGui::NextColumn();
    ImGui::Text("Input Power");
    ImGui::NextColumn();
    ImGui::Text("Input Volt");
    ImGui::NextColumn();
    ImGui::Text("Output Volt");
    ImGui::NextColumn();
    ImGui::Text("Input Current");
    ImGui::NextColumn();
    ImGui::Separator();
    for (const auto& psu : psu_status_) {
        ImGui::Text("%d", psu.id);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.present);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.power_status);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.temperature);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.pwr_in_watts);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.input_power);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.input_volt);
        ImGui::NextColumn();
        ImGui::Text("%f", psu.output_volt / 100.f);
        ImGui::NextColumn();
        ImGui::Text("%d", psu.input_current);
        ImGui::NextColumn();
    }
}

inline void draw_page() {
    ImGui::Begin("Page");
    draw_running_time();
    ImGui::Separator();
    // 绘制可折叠
    if (ImGui::CollapsingHeader("Fan Infos")) {
        draw_fan_infos();
    }
    if (ImGui::CollapsingHeader("PSU Infos")) {
        draw_psu_infos();
    }
    ImGui::End();
}
